@@ -56,6 +56,29 @@ uint32_t grn_gtick;
 int grn_uyield_count = 0;
 #endif
 
+void
+grn_sleep(uint32_t seconds)
+{
+#ifdef WIN32
+  Sleep(seconds * 1000);
+#else  // WIN32
+  sleep(seconds);
+#endif  // WIN32
+}
+
+void
+grn_nanosleep(uint64_t nanoseconds)
+{
+#ifdef WIN32
+  Sleep((DWORD)(nanoseconds / 1000000));
+#else  // WIN32
+  struct timespec interval;
+  interval.tv_sec = (time_t)(nanoseconds / 1000000000);
+  interval.tv_nsec = (long)(nanoseconds % 1000000000);
+  nanosleep(&interval, NULL);
+#endif  // WIN32
+}
+
 /* fixme by 2038 */
 
 grn_rc
@@ -636,6 +659,7 @@ grn_ctx_fin(grn_ctx *ctx)
       mi.map = (void *)ctx->impl;
       grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
     }
+    ctx->impl = NULL;
   }
   ctx->stat = GRN_CTX_FIN;
   return rc;
@@ -956,7 +980,6 @@ grn_ctx_connect(grn_ctx *ctx, const char *host, int port, int flags)
     grn_com *com = grn_com_copen(ctx, NULL, host, port);
     if (com) {
       ctx->impl->com = com;
-      return GRN_SUCCESS;
     }
   }
 exit :
@@ -983,6 +1006,16 @@ grn_ctx_get_command_version(grn_ctx *ctx)
   }
 }
 
+const char *
+grn_ctx_get_mime_type(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->mime_type;
+  } else {
+    return NULL;
+  }
+}
+
 grn_rc
 grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version)
 {
@@ -1159,6 +1192,9 @@ get_command_version(grn_ctx *ctx, const char *p, const char *pe)
 #define OUTPUT_TYPE_LEN     (sizeof(OUTPUT_TYPE) - 1)
 #define COMMAND_VERSION_LEN (sizeof(COMMAND_VERSION) - 1)
 
+#define HTTP_QUERY_PAIR_DELIMITER   "="
+#define HTTP_QUERY_PAIRS_DELIMITERS "&;"
+
 static inline int
 command_proc_p(grn_obj *expr)
 {
@@ -1182,18 +1218,18 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
     while (p < e) {
       int l;
       GRN_BULK_REWIND(&buf);
-      p = grn_text_cgidec(ctx, &buf, p, e, '=');
+      p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIR_DELIMITER);
       v = GRN_TEXT_VALUE(&buf);
       l = GRN_TEXT_LEN(&buf);
       if (l == OUTPUT_TYPE_LEN && !memcmp(v, OUTPUT_TYPE, OUTPUT_TYPE_LEN)) {
         GRN_BULK_REWIND(&buf);
-        p = grn_text_cgidec(ctx, &buf, p, e, '&');
+        p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
         v = GRN_TEXT_VALUE(&buf);
         get_content_mime_type(ctx, v, GRN_BULK_CURR(&buf));
       } else if (l == COMMAND_VERSION_LEN &&
                  !memcmp(v, COMMAND_VERSION, COMMAND_VERSION_LEN)) {
         GRN_BULK_REWIND(&buf);
-        p = grn_text_cgidec(ctx, &buf, p, e, '&');
+        p = grn_text_cgidec(ctx, &buf, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
         get_command_version(ctx, GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf));
         if (ctx->rc) { goto exit; }
       } else {
@@ -1201,7 +1237,7 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
           val = &buf;
         }
         grn_obj_reinit(ctx, val, GRN_DB_TEXT, 0);
-        p = grn_text_cgidec(ctx, val, p, e, '&');
+        p = grn_text_cgidec(ctx, val, p, e, HTTP_QUERY_PAIRS_DELIMITERS);
       }
     }
     ctx->impl->curr_expr = expr;
@@ -1281,7 +1317,7 @@ grn_ctx_qe_exec(grn_ctx *ctx, const char *str, uint32_t str_len)
     grn_text_unesc_tok(ctx, &buf, str, str + str_len, &tok_type);
     if (GRN_TEXT_LEN(&buf)) {
       ERR(GRN_INVALID_ARGUMENT, "invalid command name: %.*s",
-          GRN_TEXT_LEN(&buf), GRN_TEXT_VALUE(&buf));
+          (int)GRN_TEXT_LEN(&buf), GRN_TEXT_VALUE(&buf));
     }
   }
 exit :
@@ -1365,7 +1401,7 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
         ctx->impl->mime_type = "application/json";
         ctx->impl->output_type = GRN_CONTENT_JSON;
         grn_timeval_now(ctx, &ctx->impl->tv);
-        GRN_LOG(ctx, GRN_LOG_NONE, "%08x|>%.*s", (intptr_t)ctx, str_len, str);
+        GRN_LOG(ctx, GRN_LOG_NONE, "%p|>%.*s", ctx, str_len, str);
         if (str_len && *str == '/') {
           expr = grn_ctx_qe_exec_uri(ctx, str + 1, str_len - 1);
         } else {
@@ -1373,7 +1409,9 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
         }
       }
       if (ctx->stat == GRN_CTX_QUITTING) { ctx->stat = GRN_CTX_QUIT; }
-      if (!ctx->impl->qe_next) {
+      if (ctx->impl->qe_next) {
+        ERRCLR(ctx);
+      } else {
         LAP("<", "rc=%d", ctx->rc);
       }
     output :
@@ -1819,7 +1857,7 @@ grn_ctx_free(grn_ctx *ctx, void *ptr,
     int32_t *header = &((int32_t *)ptr)[-2];
 
     if (header[0] >= GRN_CTX_N_SEGMENTS) {
-      ERR(GRN_INVALID_ARGUMENT,"invalid ptr passed. ptr=%p seg=%zu", ptr, *header);
+      ERR(GRN_INVALID_ARGUMENT,"invalid ptr passed. ptr=%p seg=%d", ptr, *header);
       goto exit;
     }
     /*
@@ -2116,7 +2154,9 @@ grn_calloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
       grn_alloc_info_add(res);
     } else {
       if (!(res = calloc(size, 1))) {
-        MERR("calloc fail (%d)=%p (%s:%d) <%d>", size, res, file, line, alloc_count);
+        MERR("calloc fail (%" GRN_FMT_LLU ")=%p (%s:%d) <%" GRN_FMT_LLU ">",
+             (unsigned long long int)size, res, file, line,
+             (unsigned long long int)alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
         grn_alloc_info_add(res);
