@@ -421,13 +421,35 @@ void grn_expr_pack(grn_ctx *ctx, grn_obj *buf, grn_obj *expr);
 GRN_API grn_rc grn_expr_inspect(grn_ctx *ctx, grn_obj *buf, grn_obj *expr);
 grn_obj *grn_expr_open(grn_ctx *ctx, grn_obj_spec *spec, const uint8_t *p, const uint8_t *pe);
 
-grn_obj *grn_table_create_for_group(grn_ctx *ctx, const char *name, unsigned int name_size,
-                                    const char *path, grn_obj_flags flags,
-                                    grn_obj *group_key, grn_obj *value_type);
+GRN_API grn_obj *grn_table_create_for_group(grn_ctx *ctx,
+                                            const char *name,
+                                            unsigned int name_size,
+                                            const char *path,
+                                            grn_obj_flags flags,
+                                            grn_obj *group_key,
+                                            grn_obj *value_type);
 
 #define KEY_NAME "_key"
 #define ID_NAME "_id"
 
+GRN_API void grn_load_(grn_ctx *ctx, grn_content_type input_type,
+                       const char *table, unsigned int table_len,
+                       const char *columns, unsigned int columns_len,
+                       const char *values, unsigned int values_len,
+                       const char *ifexists, unsigned int ifexists_len,
+                       const char *each, unsigned int each_len,
+                       uint32_t emit_level);
+
+GRN_API grn_rc grn_table_group_with_range_gap(grn_ctx *ctx, grn_obj *table,
+                                              grn_table_sort_key *group_key,
+                                              grn_obj *result_set,
+                                              uint32_t range_gap);
+
+GRN_API grn_rc grn_column_filter(grn_ctx *ctx, grn_obj *column,
+                                 grn_operator op,
+                                 grn_obj *value, grn_obj *result_set,
+                                 grn_operator set_op);
+
 #ifdef __cplusplus
 }
 #endif
