@@ -32,33 +32,41 @@ extern "C" {
 /**** grn_tiny_array ****/
 
 /*
- * grn_tiny_array_init() accepts a logical OR of these flags.
+ * grn_tiny_array_init() accepts a logical OR of the following flags.
+ * Note that other flags, such as (1 << 30), will be ignored.
  *
- * GRN_TINY_ARRAY_CLEAR: 
- *   specifies to initialize a new block with zeros.
- * GRN_TINY_ARRAY_THREADSAFE:
- *   specifies to create a critical section when allocating memory.
- * GRN_TINY_ARRAY_USE_MALLOC:
- *   specifies to use GRN_MALLOC/CALLOC/FREE instead of GRN_CTX_ALLOC/FREE.
+ * - GRN_TINY_ARRAY_CLEAR specifies to initialize a new block with zeros.
+ *   It is valid only iff specified with GRN_TINY_ARRAY_USE_MALLOC.
+ * - GRN_TINY_ARRAY_THREADSAFE specifies to create a critical section when
+ *   allocating memory.
+ * - GRN_TINY_ARRAY_USE_MALLOC specifies to use GRN_MALLOC/CALLOC/FREE instead
+ *   of GRN_CTX_ALLOC/FREE.
  */
-#define GRN_TINY_ARRAY_CLEAR      (1<<0)
-#define GRN_TINY_ARRAY_THREADSAFE (1<<1)
-#define GRN_TINY_ARRAY_USE_MALLOC (1<<2)
+#define GRN_TINY_ARRAY_CLEAR      (1 << 0)
+#define GRN_TINY_ARRAY_THREADSAFE (1 << 1)
+#define GRN_TINY_ARRAY_USE_MALLOC (1 << 2)
 
 /*
- * GRN_TINY_ARRAY_W: a parameter of grn_tiny_array.
- * GRN_TINY_ARRAY_R: the offset to a block.
- * GRN_TINY_ARRAY_S: the number of elements in the first block.
- * GRN_TINY_ARRAY_N: the maximum number of blocks.
+ * - GRN_TINY_ARRAY_FACTOR is the global parameter of grn_tiny_array.
+ * - GRN_TINY_ARRAY_GET_OFFSET() returns the offset of a specified block.
+ * - GRN_TINY_ARRAY_BASE_BLOCK_SIZE is the number of elements in the first
+ *   block.
+ * - GRN_TINY_ARRAY_GET_BLOCK_SIZE() returns the number of elements in a
+ *   specified block.
+ * - GRN_TINY_ARRAY_NUM_BLOCKS is the maximum number of blocks.
  */
-#define GRN_TINY_ARRAY_W           0
-#define GRN_TINY_ARRAY_R(block_id) (1<<((block_id)<<GRN_TINY_ARRAY_W))
-#define GRN_TINY_ARRAY_S           (GRN_TINY_ARRAY_R(1)-1)
-#define GRN_TINY_ARRAY_N           (32>>GRN_TINY_ARRAY_W)
+#define GRN_TINY_ARRAY_FACTOR     0
+#define GRN_TINY_ARRAY_GET_OFFSET(block_id) \
+  (1 << ((block_id) << GRN_TINY_ARRAY_FACTOR))
+#define GRN_TINY_ARRAY_BASE_BLOCK_SIZE \
+  (GRN_TINY_ARRAY_GET_OFFSET(1) - GRN_TINY_ARRAY_GET_OFFSET(0))
+#define GRN_TINY_ARRAY_GET_BLOCK_SIZE(block_id) \
+  (GRN_TINY_ARRAY_BASE_BLOCK_SIZE * GRN_TINY_ARRAY_GET_OFFSET(block_id))
+#define GRN_TINY_ARRAY_NUM_BLOCKS (32 >> GRN_TINY_ARRAY_FACTOR)
 
 /*
- * grn_tiny_array uses several blocks to represent an array.
- * The k-th block (elements[k]) consists of 2^k elements.
+ * grn_tiny_array uses several blocks to emulate an array.
+ * The k-th block, blocks[k - 1], consists of 2^(k-1) elements.
  */
 typedef struct _grn_tiny_array grn_tiny_array;
 
@@ -67,26 +75,31 @@ struct _grn_tiny_array {
   grn_id max;
   uint16_t element_size;
   uint16_t flags;
+  void *blocks[GRN_TINY_ARRAY_NUM_BLOCKS];
   grn_critical_section lock;
-  void *elements[GRN_TINY_ARRAY_N];
 };
 
-#define GRN_TINY_ARRAY_EACH(array, head, tail, key, value, block) do {\
-  int _block_id;\
-  const grn_id _head = (head);\
-  const grn_id _tail = (tail);\
-  for (_block_id = 0, (key) = (_head);\
-       _block_id < GRN_TINY_ARRAY_N && (key) <= (_tail); _block_id++) {\
-    int _id = GRN_TINY_ARRAY_S * GRN_TINY_ARRAY_R(_block_id);\
-    if (((value) = (array)->elements[_block_id])) {\
-      for (; _id-- && (key) <= (_tail);\
-           (key)++, (value) = (void *)((byte *)(value) + (array)->element_size)) {\
-        block\
-      }\
-    } else {\
-      (key) += _id;\
-    }\
-  }\
+#define GRN_TINY_ARRAY_EACH(array, head, tail, key, value, block) do { \
+  int _block_id; \
+  const grn_id _head = (head); \
+  const grn_id _tail = (tail); \
+  for (_block_id = 0, (key) = (_head); \
+       _block_id < GRN_TINY_ARRAY_NUM_BLOCKS && (key) <= (_tail); \
+       _block_id++) { \
+    int _id = GRN_TINY_ARRAY_GET_BLOCK_SIZE(_block_id); \
+    (value) = (array)->blocks[_block_id]; \
+    if (value) { \
+      while (_id-- && (key) <= (_tail)) { \
+        { \
+          block \
+        } \
+        (key)++; \
+        (value) = (void *)((byte *)(value) + (array)->element_size); \
+      } \
+    } else { \
+      (key) += _id; \
+    } \
+  } \
 } while (0)
 
 GRN_API void grn_tiny_array_init(grn_ctx *ctx, grn_tiny_array *array,
@@ -96,10 +109,26 @@ GRN_API void *grn_tiny_array_at(grn_tiny_array *array, grn_id id);
 GRN_API grn_id grn_tiny_array_id(grn_tiny_array *array,
                                  const void *element_address);
 
+/**** grn_tiny_bitmap ****/
+
+typedef struct _grn_tiny_bitmap grn_tiny_bitmap;
+
+struct _grn_tiny_bitmap {
+  grn_ctx *ctx;
+  void *blocks[GRN_TINY_ARRAY_NUM_BLOCKS];
+};
+
 /**** grn_array ****/
 
 #define GRN_ARRAY_TINY        (0x01<<6)
 
+/*
+ * grn_array uses grn_io or grn_tiny_array to represent an array.
+ *
+ * To create a grn_tiny_array-based grn_array, specify the GRN_ARRAY_TINY flag
+ * to grn_array_create(). Note that a grn_tiny_array-based grn_array is not
+ * backed by a file.
+ */
 struct _grn_array {
   grn_db_obj obj;
   grn_ctx *ctx;
@@ -108,16 +137,18 @@ struct _grn_array {
   grn_table_sort_key *keys;
   uint32_t *n_garbages;
   uint32_t *n_entries;
-  /* portions for io_array */
+
+  /* For grn_io_array. */
   grn_io *io;
   struct grn_array_header *header;
   uint32_t *lock;
-  /* portions for tiny_array */
-  uint32_t n_garbages_;
-  uint32_t n_entries_;
+
+  /* For grn_tiny_array. */
+  uint32_t n_garbages_buf;
+  uint32_t n_entries_buf;
   grn_id garbages;
-  grn_tiny_array a;
-  grn_tiny_array bitmap;
+  grn_tiny_array array;
+  grn_tiny_bitmap bitmap;
 };
 
 struct _grn_array_cursor {
@@ -136,6 +167,22 @@ grn_rc grn_array_truncate(grn_ctx *ctx, grn_array *array);
 grn_rc grn_array_copy_sort_key(grn_ctx *ctx, grn_array *array,
                                grn_table_sort_key *keys, int n_keys);
 
+/* grn_table_queue */
+
+typedef struct _grn_table_queue grn_table_queue;
+
+struct _grn_table_queue {
+  grn_mutex mutex;
+  grn_cond cond;
+  grn_id head;
+  grn_id tail;
+  grn_id cap;
+};
+
+GRN_API void grn_array_queue_lock_clear(grn_ctx *ctx, grn_array *array);
+GRN_API void grn_array_clear_curr_rec(grn_ctx *ctx, grn_array *array);
+GRN_API grn_table_queue *grn_array_queue(grn_ctx *ctx, grn_array *array);
+
 /**** grn_hash ****/
 
 #define GRN_HASH_TINY         (0x01<<6)
@@ -152,7 +199,8 @@ struct _grn_hash {
   uint32_t *n_entries;
   uint32_t *max_offset;
   grn_obj *tokenizer;
-  /* portions for io_hash */
+
+  /* For grn_io_hash. */
   grn_io *io;
   struct grn_hash_header *header;
   uint32_t *lock;
@@ -168,16 +216,18 @@ struct _grn_hash {
   // int limit;
   // void *userdata;
   // grn_id subrec_id;
-  /* portions for tiny_hash */
+
+  /* For grn_tiny_hash. */
   uint32_t max_offset_;
   uint32_t n_garbages_;
   uint32_t n_entries_;
   grn_id *index;
   grn_id garbages;
   grn_tiny_array a;
-  grn_tiny_array bitmap;
+  grn_tiny_bitmap bitmap;
 };
 
+/* Header of grn_io_hash. */
 struct grn_hash_header {
   uint32_t flags;
   grn_encoding encoding;
@@ -194,6 +244,7 @@ struct grn_hash_header {
   uint32_t lock;
   uint32_t reserved[16];
   grn_id garbages[GRN_HASH_MAX_KEY_SIZE];
+  grn_table_queue queue;
 };
 
 struct _grn_hash_cursor {
