@@ -15,6 +15,8 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#include <string.h>
+
 #include "ctx.h"
 #include "db.h"
 #include "output.h"
@@ -84,11 +86,14 @@ parse_set_operator_value(grn_ctx *ctx, grn_obj *text)
     if (memcmp("and", GRN_TEXT_VALUE(text), 3) == 0) {
       value = GRN_OP_AND;
     } else if (memcmp("but", GRN_TEXT_VALUE(text), 3) == 0) {
-      value = GRN_OP_BUT;
+      value = GRN_OP_AND_NOT;
     }
   } else if (GRN_TEXT_LEN(text) == 6 &&
              memcmp("adjust", GRN_TEXT_VALUE(text), 6) == 0) {
     value = GRN_OP_ADJUST;
+  } else if (GRN_TEXT_LEN(text) == 7 &&
+             memcmp("and_not", GRN_TEXT_VALUE(text), 7) == 0) {
+    value = GRN_OP_AND_NOT;
   }
   return value;
 }
