@@ -245,459 +245,10 @@ typedef enum {
   grn_http_request_type_post
 } grn_http_request_type;
 
-typedef enum {
-  XML_START,
-  XML_START_ELEMENT,
-  XML_END_ELEMENT,
-  XML_TEXT
-} xml_status;
-
-typedef enum {
-  XML_PLACE_NONE,
-  XML_PLACE_COLUMN,
-  XML_PLACE_HIT
-} xml_place;
-
-static char *
-transform_xml_next_column(grn_obj *columns, int n)
-{
-  char *column = GRN_TEXT_VALUE(columns);
-  while (n--) {
-    while (*column) {
-      column++;
-    }
-    column++;
-  }
-  return column;
-}
-
 static void
-transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
+output_envelope(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj *foot)
 {
-  char *s, *e;
-  xml_status status = XML_START;
-  xml_place place = XML_PLACE_NONE;
-  grn_obj buf, name, columns, *expr;
-  unsigned int len;
-  int offset = 0, limit = 0, record_n = 0;
-  int column_n = 0, column_text_n = 0, result_set_n = -1;
-  int in_vector = 0, first_vector_element = 0;
-
-  s = GRN_TEXT_VALUE(output);
-  e = GRN_BULK_CURR(output);
-  GRN_TEXT_INIT(&buf, 0);
-  GRN_TEXT_INIT(&name, 0);
-  GRN_TEXT_INIT(&columns, 0);
-
-  expr = ctx->impl->curr_expr;
-
-#define EQUAL_NAME_P(_name) \
-  (GRN_TEXT_LEN(&name) == strlen(_name) && \
-   !memcmp(GRN_TEXT_VALUE(&name), _name, strlen(_name)))
-
-  while (s < e) {
-    switch (*s) {
-    case '<' :
-      s++;
-      switch (*s) {
-      case '/' :
-        status = XML_END_ELEMENT;
-        s++;
-        break;
-      default :
-        status = XML_START_ELEMENT;
-        break;
-      }
-      GRN_BULK_REWIND(&name);
-      break;
-    case '>' :
-      switch (status) {
-      case XML_START_ELEMENT :
-        if (EQUAL_NAME_P("COLUMN")) {
-          place = XML_PLACE_COLUMN;
-          column_text_n = 0;
-        } else if (EQUAL_NAME_P("HIT")) {
-          place = XML_PLACE_HIT;
-          column_n = 0;
-          if (result_set_n == 0) {
-            GRN_TEXT_PUTS(ctx, transformed, "<HIT NO=\"");
-            grn_text_itoa(ctx, transformed, record_n++);
-            GRN_TEXT_PUTS(ctx, transformed, "\">\n");
-          } else {
-            GRN_TEXT_PUTS(ctx, transformed, "<NAVIGATIONELEMENT ");
-          }
-        } else if (EQUAL_NAME_P("RESULTSET")) {
-          GRN_BULK_REWIND(&columns);
-          result_set_n++;
-          if (result_set_n == 0) {
-          } else {
-            GRN_TEXT_PUTS(ctx, transformed, "<NAVIGATIONENTRY>\n");
-          }
-        } else if (EQUAL_NAME_P("VECTOR")) {
-          char *c = transform_xml_next_column(&columns, column_n++);
-          in_vector = 1;
-          first_vector_element = 1;
-          GRN_TEXT_PUTS(ctx, transformed, "<FIELD NAME=\"");
-          GRN_TEXT_PUTS(ctx, transformed, c);
-          GRN_TEXT_PUTS(ctx, transformed, "\">");
-        }
-        break;
-      case XML_END_ELEMENT :
-        if (EQUAL_NAME_P("HIT")) {
-          place = XML_PLACE_NONE;
-          if (result_set_n == 0) {
-            GRN_TEXT_PUTS(ctx, transformed, "</HIT>\n");
-          } else {
-            GRN_TEXT_PUTS(ctx, transformed, "/>\n");
-          }
-        } else if (EQUAL_NAME_P("RESULTSET")) {
-          place = XML_PLACE_NONE;
-          if (result_set_n == 0) {
-            GRN_TEXT_PUTS(ctx, transformed, "</RESULTSET>\n");
-          } else {
-            GRN_TEXT_PUTS(ctx, transformed,
-                          "</NAVIGATIONELEMENTS>\n"
-                          "</NAVIGATIONENTRY>\n");
-          }
-        } else if (EQUAL_NAME_P("RESULT")) {
-          GRN_TEXT_PUTS(ctx, transformed,
-                        "</RESULTPAGE>\n"
-                        "</SEGMENT>\n"
-                        "</SEGMENTS>\n");
-        } else if (EQUAL_NAME_P("VECTOR")) {
-          in_vector = 0;
-          first_vector_element = 0;
-          GRN_TEXT_PUTS(ctx, transformed, "</FIELD>\n");
-        } else {
-          switch (place) {
-          case XML_PLACE_HIT :
-            if (result_set_n == 0) {
-              if (!in_vector) {
-                char *c = transform_xml_next_column(&columns, column_n++);
-                GRN_TEXT_PUTS(ctx, transformed, "<FIELD NAME=\"");
-                GRN_TEXT_PUTS(ctx, transformed, c);
-                GRN_TEXT_PUTS(ctx, transformed, "\">");
-              }
-              if (in_vector && !first_vector_element) {
-                GRN_TEXT_PUTS(ctx, transformed, ", ");
-              }
-              GRN_TEXT_PUT(ctx, transformed,
-                           GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
-              if (!in_vector) {
-                GRN_TEXT_PUTS(ctx, transformed, "</FIELD>\n");
-              }
-            } else {
-              char *c = transform_xml_next_column(&columns, column_n++);
-              GRN_TEXT_PUTS(ctx, transformed, c);
-              GRN_TEXT_PUTS(ctx, transformed, "=\"");
-              GRN_TEXT_PUT(ctx, transformed,
-                           GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
-              GRN_TEXT_PUTS(ctx, transformed, "\" ");
-            }
-            first_vector_element = 0;
-            break;
-          default :
-            if (EQUAL_NAME_P("NHITS")) {
-              if (result_set_n == 0) {
-                uint32_t nhits;
-                grn_obj *offset_value, *limit_value;
-
-                nhits = grn_atoui(GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf),
-                                  NULL);
-                offset_value = grn_expr_get_var(ctx, expr,
-                                                "offset", strlen("offset"));
-                limit_value = grn_expr_get_var(ctx, expr,
-                                               "limit", strlen("limit"));
-                if (GRN_TEXT_LEN(offset_value)) {
-                  offset = grn_atoi(GRN_TEXT_VALUE(offset_value),
-                                    GRN_BULK_CURR(offset_value),
-                                    NULL);
-                } else {
-                  offset = 0;
-                }
-                if (GRN_TEXT_LEN(limit_value)) {
-                  limit = grn_atoi(GRN_TEXT_VALUE(limit_value),
-                                   GRN_BULK_CURR(limit_value),
-                                   NULL);
-                } else {
-#define DEFAULT_LIMIT 10
-                  limit = DEFAULT_LIMIT;
-#undef DEFAULT_LIMIT
-                }
-                grn_normalize_offset_and_limit(ctx, nhits, &offset, &limit);
-                record_n = offset + 1;
-                GRN_TEXT_PUTS(ctx, transformed,
-                              "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
-                              "<SEGMENTS>\n"
-                              "<SEGMENT>\n"
-                              "<RESULTPAGE>\n"
-                              "<RESULTSET OFFSET=\"");
-                grn_text_lltoa(ctx, transformed, offset);
-                GRN_TEXT_PUTS(ctx, transformed, "\" LIMIT=\"");
-                grn_text_lltoa(ctx, transformed, limit);
-                GRN_TEXT_PUTS(ctx, transformed, "\" NHITS=\"");
-                grn_text_lltoa(ctx, transformed, nhits);
-                GRN_TEXT_PUTS(ctx, transformed, "\">\n");
-              } else {
-                GRN_TEXT_PUTS(ctx, transformed,
-                              "<NAVIGATIONELEMENTS COUNT=\"");
-                GRN_TEXT_PUT(ctx, transformed,
-                             GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
-                GRN_TEXT_PUTS(ctx, transformed,
-                              "\">\n");
-              }
-            } else if (EQUAL_NAME_P("TEXT")) {
-              switch (place) {
-              case XML_PLACE_COLUMN :
-                if (column_text_n == 0) {
-                  GRN_TEXT_PUT(ctx, &columns,
-                               GRN_TEXT_VALUE(&buf), GRN_TEXT_LEN(&buf));
-                  GRN_TEXT_PUTC(ctx, &columns, '\0');
-                }
-                column_text_n++;
-                break;
-              default :
-                break;
-              }
-            }
-          }
-        }
-      default :
-        break;
-      }
-      s++;
-      GRN_BULK_REWIND(&buf);
-      status = XML_TEXT;
-      break;
-    default :
-      len = grn_charlen(ctx, s, e);
-      switch (status) {
-      case XML_START_ELEMENT :
-      case XML_END_ELEMENT :
-        GRN_TEXT_PUT(ctx, &name, s, len);
-        break;
-      default :
-        GRN_TEXT_PUT(ctx, &buf, s, len);
-        break;
-      }
-      s += len;
-      break;
-    }
-  }
-#undef EQUAL_NAME_P
-
-  GRN_OBJ_FIN(ctx, &buf);
-  GRN_OBJ_FIN(ctx, &name);
-  GRN_OBJ_FIN(ctx, &columns);
-}
-
-#ifdef WITH_MESSAGE_PACK
-typedef struct {
-  grn_ctx *ctx;
-  grn_obj *buffer;
-} msgpack_writer_ctx;
-
-static inline int
-msgpack_buffer_writer(void* data, const char* buf, unsigned int len)
-{
-  msgpack_writer_ctx *writer_ctx = (msgpack_writer_ctx *)data;
-  return grn_bulk_write(writer_ctx->ctx, writer_ctx->buffer, buf, len);
-}
-#endif
-
-static void
-print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj *foot)
-{
-  double started, finished, elapsed;
-
-  grn_timeval tv_now;
-  grn_timeval_now(ctx, &tv_now);
-  started = ctx->impl->tv.tv_sec;
-  started += ctx->impl->tv.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
-  finished = tv_now.tv_sec;
-  finished += tv_now.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
-  elapsed = finished - started;
-
-  switch (ctx->impl->output_type) {
-  case GRN_CONTENT_JSON:
-    GRN_TEXT_PUTS(ctx, head, "[[");
-    grn_text_itoa(ctx, head, rc);
-    GRN_TEXT_PUTC(ctx, head, ',');
-    grn_text_ftoa(ctx, head, started);
-    GRN_TEXT_PUTC(ctx, head, ',');
-    grn_text_ftoa(ctx, head, elapsed);
-    if (rc != GRN_SUCCESS) {
-      GRN_TEXT_PUTC(ctx, head, ',');
-      grn_text_esc(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
-      if (ctx->errfunc && ctx->errfile) {
-        grn_obj *command;
-        /* TODO: output backtrace */
-        GRN_TEXT_PUTS(ctx, head, ",[[");
-        grn_text_esc(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
-        GRN_TEXT_PUTC(ctx, head, ',');
-        grn_text_esc(ctx, head, ctx->errfile, strlen(ctx->errfile));
-        GRN_TEXT_PUTC(ctx, head, ',');
-        grn_text_itoa(ctx, head, ctx->errline);
-        if (input_path && (command = GRN_CTX_USER_DATA(ctx)->ptr)) {
-          GRN_TEXT_PUTC(ctx, head, ',');
-          grn_text_esc(ctx, head, input_path, strlen(input_path));
-          GRN_TEXT_PUTC(ctx, head, ',');
-          grn_text_itoa(ctx, head, number_of_lines);
-          GRN_TEXT_PUTC(ctx, head, ',');
-          grn_text_esc(ctx, head, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
-        }
-        GRN_TEXT_PUTS(ctx, head, "]]");
-      }
-    }
-    GRN_TEXT_PUTC(ctx, head, ']');
-    if (GRN_TEXT_LEN(body)) { GRN_TEXT_PUTC(ctx, head, ','); }
-    GRN_TEXT_PUTC(ctx, foot, ']');
-    break;
-  case GRN_CONTENT_TSV:
-    grn_text_itoa(ctx, head, rc);
-    GRN_TEXT_PUTC(ctx, head, '\t');
-    grn_text_ftoa(ctx, head, started);
-    GRN_TEXT_PUTC(ctx, head, '\t');
-    grn_text_ftoa(ctx, head, elapsed);
-    if (rc != GRN_SUCCESS) {
-      GRN_TEXT_PUTC(ctx, head, '\t');
-      grn_text_esc(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
-      if (ctx->errfunc && ctx->errfile) {
-        /* TODO: output backtrace */
-        GRN_TEXT_PUTC(ctx, head, '\t');
-        grn_text_esc(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
-        GRN_TEXT_PUTC(ctx, head, '\t');
-        grn_text_esc(ctx, head, ctx->errfile, strlen(ctx->errfile));
-        GRN_TEXT_PUTC(ctx, head, '\t');
-        grn_text_itoa(ctx, head, ctx->errline);
-      }
-    }
-    GRN_TEXT_PUTS(ctx, head, "\n");
-    GRN_TEXT_PUTS(ctx, foot, "\nEND");
-    break;
-  case GRN_CONTENT_XML:
-    {
-      char buf[GRN_TABLE_MAX_KEY_SIZE];
-      int is_select = 0;
-      if (!rc && ctx->impl->curr_expr) {
-        int len = grn_obj_name(ctx, ctx->impl->curr_expr,
-                               buf, GRN_TABLE_MAX_KEY_SIZE);
-        buf[len] = '\0';
-        is_select = strcmp(buf, "select") == 0;
-      }
-      if (is_select) {
-        grn_obj transformed;
-        GRN_TEXT_INIT(&transformed, 0);
-        transform_xml(ctx, body, &transformed);
-        GRN_TEXT_SET(ctx, body,
-                     GRN_TEXT_VALUE(&transformed), GRN_TEXT_LEN(&transformed));
-        GRN_OBJ_FIN(ctx, &transformed);
-      } else {
-        GRN_TEXT_PUTS(ctx, head, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<RESULT CODE=\"");
-        grn_text_itoa(ctx, head, rc);
-        GRN_TEXT_PUTS(ctx, head, "\" UP=\"");
-        grn_text_ftoa(ctx, head, started);
-        GRN_TEXT_PUTS(ctx, head, "\" ELAPSED=\"");
-        grn_text_ftoa(ctx, head, elapsed);
-        GRN_TEXT_PUTS(ctx, head, "\">\n");
-        if (rc != GRN_SUCCESS) {
-          GRN_TEXT_PUTS(ctx, head, "<ERROR>");
-          grn_text_escape_xml(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
-          if (ctx->errfunc && ctx->errfile) {
-            /* TODO: output backtrace */
-            GRN_TEXT_PUTS(ctx, head, "<INFO FUNC=\"");
-            grn_text_escape_xml(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
-            GRN_TEXT_PUTS(ctx, head, "\" FILE=\"");
-            grn_text_escape_xml(ctx, head, ctx->errfile, strlen(ctx->errfile));
-            GRN_TEXT_PUTS(ctx, head, "\" LINE=\"");
-            grn_text_itoa(ctx, head, ctx->errline);
-            GRN_TEXT_PUTS(ctx, head, "\"/>");
-          }
-          GRN_TEXT_PUTS(ctx, head, "</ERROR>");
-        }
-        GRN_TEXT_PUTS(ctx, foot, "\n</RESULT>");
-      }
-    }
-    break;
-  case GRN_CONTENT_MSGPACK:
-#ifdef WITH_MESSAGE_PACK
-    {
-      msgpack_writer_ctx head_writer_ctx;
-      msgpack_packer header_packer;
-      int header_size;
-
-      head_writer_ctx.ctx = ctx;
-      head_writer_ctx.buffer = head;
-      msgpack_packer_init(&header_packer, &head_writer_ctx, msgpack_buffer_writer);
-
-       /* [HEAD, (BODY)] */
-      if (GRN_TEXT_LEN(body) > 0) {
-        msgpack_pack_array(&header_packer, 2);
-      } else {
-        msgpack_pack_array(&header_packer, 1);
-      }
-
-      /* HEAD := [rc, started, elapsed, (error, (ERROR DETAIL))] */
-      header_size = 3;
-      if (rc != GRN_SUCCESS) {
-        header_size++;
-        if (ctx->errfunc && ctx->errfile) {
-          header_size++;
-        }
-      }
-      msgpack_pack_array(&header_packer, header_size);
-      msgpack_pack_int(&header_packer, rc);
-
-      msgpack_pack_double(&header_packer, started);
-      msgpack_pack_double(&header_packer, elapsed);
-
-      if (rc != GRN_SUCCESS) {
-        msgpack_pack_raw(&header_packer, strlen(ctx->errbuf));
-        msgpack_pack_raw_body(&header_packer, ctx->errbuf, strlen(ctx->errbuf));
-        if (ctx->errfunc && ctx->errfile) {
-          grn_obj *command = GRN_CTX_USER_DATA(ctx)->ptr;
-          int error_detail_size;
-
-          /* ERROR DETAIL := [[errfunc, errfile, errline,
-                               (input_path, number_of_lines, command)]] */
-          /* TODO: output backtrace */
-          msgpack_pack_array(&header_packer, 1);
-          error_detail_size = 3;
-          if (command) {
-            error_detail_size += 3;
-          }
-          msgpack_pack_array(&header_packer, error_detail_size);
-
-          msgpack_pack_raw(&header_packer, strlen(ctx->errfunc));
-          msgpack_pack_raw_body(&header_packer, ctx->errfunc, strlen(ctx->errfunc));
-
-          msgpack_pack_raw(&header_packer, strlen(ctx->errfile));
-          msgpack_pack_raw_body(&header_packer, ctx->errfile, strlen(ctx->errfile));
-
-          msgpack_pack_int(&header_packer, ctx->errline);
-
-          if (command) {
-            if (input_path) {
-              msgpack_pack_raw(&header_packer, strlen(input_path));
-              msgpack_pack_raw_body(&header_packer, input_path, strlen(input_path));
-            } else {
-              msgpack_pack_raw(&header_packer, 7);
-              msgpack_pack_raw_body(&header_packer, "(stdin)", 7);
-            }
-
-            msgpack_pack_int(&header_packer, number_of_lines);
-
-            msgpack_pack_raw(&header_packer, GRN_TEXT_LEN(command));
-            msgpack_pack_raw_body(&header_packer, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
-          }
-        }
-      }
-    }
-#endif
-    break;
-  case GRN_CONTENT_NONE:
-    break;
-  }
+  grn_output_envelope(ctx, rc, head, body, foot, input_path, number_of_lines);
 }
 
 static void
@@ -711,7 +262,7 @@ s_output(grn_ctx *ctx, int flags, void *arg)
       grn_obj head, foot;
       GRN_TEXT_INIT(&head, 0);
       GRN_TEXT_INIT(&foot, 0);
-      print_return_code(ctx, ctx->rc, &head, buf, &foot);
+      output_envelope(ctx, ctx->rc, &head, buf, &foot);
       fwrite(GRN_TEXT_VALUE(&head), 1, GRN_TEXT_LEN(&head), stream);
       fwrite(GRN_TEXT_VALUE(buf), 1, GRN_TEXT_LEN(buf), stream);
       fwrite(GRN_TEXT_VALUE(&foot), 1, GRN_TEXT_LEN(&foot), stream);
@@ -785,7 +336,7 @@ c_output(grn_ctx *ctx)
       GRN_TEXT_INIT(&body, GRN_OBJ_DO_SHALLOW_COPY);
       GRN_TEXT_INIT(&foot, 0);
       GRN_TEXT_SET(ctx, &body, str, str_len);
-      print_return_code(ctx, ctx->rc, &head, &body, &foot);
+      output_envelope(ctx, ctx->rc, &head, &body, &foot);
       fwrite(GRN_TEXT_VALUE(&head), 1, GRN_TEXT_LEN(&head), output);
       fwrite(GRN_TEXT_VALUE(&body), 1, GRN_TEXT_LEN(&body), output);
       fwrite(GRN_TEXT_VALUE(&foot), 1, GRN_TEXT_LEN(&foot), output);
@@ -985,8 +536,6 @@ run_server(grn_ctx *ctx, grn_obj *db, grn_com_event *ev,
   return exit_code;
 }
 
-#define JSON_CALLBACK_PARAM "callback"
-
 typedef struct {
   grn_obj body;
   grn_msg *msg;
@@ -999,44 +548,30 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   ht_context *hc = (ht_context *)arg;
   grn_sock fd = hc->msg->u.fd;
   grn_obj *body = &hc->body;
-  const char *mime_type = ctx->impl->mime_type;
   grn_obj head, foot, *outbuf = ctx->impl->outbuf;
   if (!(flags & GRN_CTX_TAIL)) { return; }
   GRN_TEXT_INIT(&head, 0);
   GRN_TEXT_INIT(&foot, 0);
   if (!expr_rc) {
-    grn_obj *expr = ctx->impl->curr_expr;
-    grn_obj *jsonp_func = NULL;
-    if (expr) {
-      jsonp_func = grn_expr_get_var(ctx, expr, JSON_CALLBACK_PARAM,
-                                    strlen(JSON_CALLBACK_PARAM));
-    }
-    if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
-      GRN_TEXT_PUT(ctx, &head, GRN_TEXT_VALUE(jsonp_func), GRN_TEXT_LEN(jsonp_func));
-      GRN_TEXT_PUTC(ctx, &head, '(');
-      print_return_code(ctx, expr_rc, &head, outbuf, &foot);
-      GRN_TEXT_PUTS(ctx, &foot, ");");
-    } else {
-      print_return_code(ctx, expr_rc, &head, outbuf, &foot);
-    }
+    output_envelope(ctx, expr_rc, &head, outbuf, &foot);
     GRN_TEXT_SETS(ctx, body, "HTTP/1.1 200 OK\r\n");
     GRN_TEXT_PUTS(ctx, body, "Connection: close\r\n");
     GRN_TEXT_PUTS(ctx, body, "Content-Type: ");
-    GRN_TEXT_PUTS(ctx, body, mime_type);
+    GRN_TEXT_PUTS(ctx, body, grn_ctx_get_mime_type(ctx));
     GRN_TEXT_PUTS(ctx, body, "\r\nContent-Length: ");
     grn_text_lltoa(ctx, body,
                    GRN_TEXT_LEN(&head) + GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot));
     GRN_TEXT_PUTS(ctx, body, "\r\n\r\n");
   } else {
     GRN_BULK_REWIND(outbuf);
-    print_return_code(ctx, expr_rc, &head, outbuf, &foot);
+    output_envelope(ctx, expr_rc, &head, outbuf, &foot);
     if (expr_rc == GRN_NO_SUCH_FILE_OR_DIRECTORY) {
       GRN_TEXT_SETS(ctx, body, "HTTP/1.1 404 Not Found\r\n");
     } else {
       GRN_TEXT_SETS(ctx, body, "HTTP/1.1 500 Internal Server Error\r\n");
     }
     GRN_TEXT_PUTS(ctx, body, "Content-Type: ");
-    GRN_TEXT_PUTS(ctx, body, mime_type);
+    GRN_TEXT_PUTS(ctx, body, grn_ctx_get_mime_type(ctx));
     GRN_TEXT_PUTS(ctx, body, "\r\n\r\n");
   }
   {
@@ -2721,7 +2256,11 @@ main(int argc, char **argv)
       }
       batchmode = GRN_TRUE;
     } else {
-      batchmode = !isatty(0);
+      if (argc - i > 1) {
+        batchmode = GRN_TRUE;
+      } else {
+        batchmode = !isatty(0);
+      }
     }
   }
 
