@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #define __STDC_LIMIT_MACROS
@@ -36,7 +36,7 @@
 namespace
 {
   void enter_api(grn_ctx *ctx) {
-    GRN_API_ENTER
+    GRN_API_ENTER;
   }
 
   int leave_api(grn_ctx *ctx) {
