@@ -13,13 +13,17 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <gcutter.h>
 #include <glib/gstdio.h>
 #include <cppcutter.h>
 
+#ifdef UINT32_MAX
+ #undef UINT32_MAX
+#endif  // UINT32_MAX
+
 #include <grn-assertions.h>
 #include <dat/trie.hpp>
 
