@@ -10,7 +10,19 @@ top_dir="$BUILD_DIR/../.."
 top_dir=$(cd -P "$top_dir" 2>/dev/null || cd "$top_dir"; pwd)
 
 if test x"$NO_MAKE" != x"yes"; then
-    make -C $top_dir > /dev/null || exit 1
+    MAKE_ARGS=
+    case `uname` in
+	Linux)
+	    MAKE_ARGS="-j$(grep '^processor' /proc/cpuinfo | wc -l)"
+	    ;;
+	Darwin)
+	    MAKE_ARGS="-j$(/usr/sbin/sysctl -n hw.ncpu)"
+	    ;;
+	*)
+	    :
+	    ;;
+    esac
+    make $MAKE_ARGS -C $top_dir > /dev/null || exit 1
 fi
 
 if test -z "$CUTTER"; then
@@ -33,6 +45,9 @@ if test -z "$GROONGA_BENCHMARK"; then
 fi
 export GROONGA_BENCHMARK
 
+TZ=Asia/Tokyo
+export TZ
+
 CUTTER_ARGS=
 CUTTER_WRAPPER=
 if test x"$CUTTER_DEBUG" = x"yes"; then
@@ -93,7 +108,8 @@ if test "$NO_RUBY" != "yes" -a -n "$RUBY"; then
     if ! type bundle > /dev/null; then
 	$RUBY -S gem install bundler
     fi
-    if [ "$BUNDLE_GEMFILE" -nt "$BUNDLE_GEMFILE.lock" ]; then
+    if [ ! -e "$BUNDLE_GEMFILE.lock" -o \
+	 "$BUNDLE_GEMFILE" -nt "$BUNDLE_GEMFILE.lock" ]; then
 	$RUBY -S bundle install
     fi
     $RUBY $BASE_DIR/run-test.rb $RUBY_TEST_ARGS "$@"
